
# q-expansion of newform 170.2.g.f, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 170
weight = 2
poly_data = [1, 0, 0, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[137, [-1, 0, 0, 0]], [71, [0, 0, 1, 0]]]
aps_data = [[1, 0, 0, 0], [1, 0, -1, -1], [0, 1, 0, 2], [-1, 0, -1, 0], [-3, 0, 3, 2], [0, 0, -3, 0], [0, -4, 0, -1], [0, -3, 3, -3], [0, 4, 0, 0], [0, -1, 0, 0], [1, 9, 1, 0], [-1, 0, 1, -6], [3, 0, -3, 2], [-8, -3, 0, 3], [0, 1, 3, 1], [3, 0, 0, 0], [0, -7, 3, -7], [4, 0, -4, 3], [0, 6, -6, 6], [3, -1, 3, 0], [2, 0, -2, 9], [4, 0, -4, -6], [0, 3, 0, -3], [3, -6, 0, 6], [2, 0, -2, -3], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 4], [-2, 0, 2, 15], [-6, -5, -6, 0], [-5, 9, 0, -9], [9, 2, 9, 0], [0, 1, 6, 1], [-11, 0, -11, 0], [0, 9, 0, -9], [0, 0, -12, 0], [0, 0, 12, 0], [-4, -12, -4, 0], [-6, 0, 6, 10], [-12, 0, 12, -8], [0, 8, 12, 8], [-14, 0, 14, 0], [0, -15, 0, 15], [-10, 12, -10, 0], [-9, -8, -9, 0], [-5, 3, -5, 0], [-8, 0, 8, 0], [19, -3, 0, 3], [9, 7, 9, 0], [0, 3, 12, 3], [6, 0, -6, 7], [18, -3, 0, 3], [7, 6, 7, 0], [12, -6, 0, 6], [6, 9, 0, -9], [-9, -3, 0, 3], [-12, -1, -12, 0], [2, 6, 0, -6], [-4, 0, 4, -6], [0, -4, -9, -4], [17, -3, 17, 0], [0, 4, -15, 4], [0, 15, -12, 15], [0, 2, 0, 0], [14, -12, 14, 0], [0, -20, 0, 0], [0, 9, -3, 9], [2, 0, -2, -21], [-15, 1, -15, 0], [0, -12, -6, -12], [0, -8, 0, -8], [0, 5, 0, 5], [14, -6, 14, 0], [0, -6, 24, -6], [-8, -12, -8, 0], [9, -3, 0, 3], [0, -16, -6, -16], [8, 18, 8, 0], [-12, 0, 12, 14], [17, -6, 0, 6], [0, 0, 0, -16], [2, 15, 0, -15], [12, 0, -12, 8], [10, 3, 0, -3], [4, -24, 4, 0], [0, -14, 18, -14], [-9, 0, 9, -28], [-2, -3, 0, 3], [0, 20, -12, 20], [0, 3, -21, 3], [6, -9, 0, 9], [-15, -7, -15, 0], [-7, 6, -7, 0], [0, -7, -3, -7], [-17, 0, 17, 12], [-6, 10, -6, 0], [0, 0, 0, 0], [0, 0, 0, -16], [0, 6, -18, 6], [4, -24, 4, 0], [-13, 0, 13, 15], [0, -14, 15, -14], [12, -24, 0, 24], [0, 8, -9, 8], [1, 0, -1, -30], [0, -12, -12, -12], [-24, 0, 0, 0], [-18, 0, 0, 0], [-12, 0, 0, 0], [16, 0, -16, 18], [-1, 0, 1, -36], [0, 12, 21, 12], [6, 0, -6, 19], [10, 0, -10, -6], [0, 6, 6, 6], [-12, -22, -12, 0], [8, 0, -8, -6], [0, -23, 15, -23], [15, 28, 15, 0], [9, -15, 0, 15], [0, 0, -12, 0], [-28, -3, -28, 0], [-21, 0, 21, -2], [-21, 0, 21, -5], [1, 0, -1, 12], [-12, 0, 0, 0], [-8, -15, -8, 0], [-3, 35, -3, 0], [0, -3, -21, -3], [16, -12, 0, 12], [0, 27, 3, 27], [15, 0, -15, 4], [13, -9, 13, 0], [1, 24, 0, -24], [12, -6, 0, 6], [11, 18, 0, -18], [24, -6, 0, 6], [5, 0, -5, -9], [12, -12, 0, 12], [21, 0, -21, 20], [1, -12, 1, 0], [-6, 17, -6, 0], [2, 18, 2, 0], [6, 16, 6, 0], [14, 0, 0, 0], [-15, 0, 15, 17], [17, 0, -17, -12], [-12, 19, -12, 0], [0, 9, -33, 9], [0, 22, -24, 22], [17, 12, 17, 0], [-24, -22, -24, 0], [0, -9, 6, -9], [21, 0, -21, -2], [-7, -21, -7, 0], [-24, 0, 24, -16], [-10, 15, 0, -15], [-21, 0, 21, 8], [-26, 15, 0, -15], [24, 0, -24, -13], [-27, 0, 27, 1], [0, 1, 6, 1], [-14, 12, 0, -12], [0, 11, -15, 11], [12, 9, 0, -9], [3, 0, -3, 58], [19, -3, 19, 0], [-1, 42, -1, 0]]
